/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "tasks/tasks.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <stdio.h>
#include "snd/soundHandler.h"
#include "ui/uiHandler.h"
#include "util/util.h"
#include "file/file.h"

extern int DBuf_init (long (*)(u32*, long), long (*)(u32*, long), int);
extern int DBuf_rerror(), DBuf_werror();
extern void DBuf_close();

static long filereader (u32*, long), soundwriter (u32*, long);
static void (*convert)(u32*, long);

static const struct File* f;
static long size, samp_fac, read, written;
static int timer, b, contd_pos, closed, read_finished, error;

int Task_play_init (int contd)
{
  const struct File* buf = File_init (Config_buffilename());
  const struct File* file = File_init (Config_filename());
  int r, c;

  error = 0;
  if (File_empty (buf) && File_empty (file)) {
  	error = 2;
  	UIHandler_msg (strcatalloc (_("Open failed: "), File_name (file)));
  	sleep (1);
  	return -1;
  }

  f = (File_empty (buf))? file : buf;
  timer = Config_iget ("TIMER");
  
  UIHandler_time (Time_i2clock (0L));
  File_begin_read (f);
  Format_init (f, contd);
  Format_get_params (&r, &b, &c);
  convert = Format_get_conversion();
  
  if (contd) 
  	File_seek (f, contd_pos);
  else
  	read = written = 0;
  	
  UIHandler_size (File_tell (f), size = File_size (f));
  SoundHandler_init (&r, &b, &c, 1);
  samp_fac = c*(b/8)*r; /* TODO: make this to what's really played */
  UIHandler_rbc (r, b, c);
  
  if (!UIHandler_is_commandline())
  	UIHandler_msg (_("Filling buffer..."));
  /* two new threads are started and the remaining takes over */
  /* error checking and keeping the clock in sync             */
  if (DBuf_init (filereader, soundwriter, samp_fac) < 0)
  	Exception_fatal ("Init of buffer failed.  Enough memory?");
  
  if (!UIHandler_is_commandline())
  	UIHandler_msg (_("Playing..."));
  read_finished = closed = 0;
  return 0;
}

void Task_play_finish (int contd)
{
  if (closed || error == 2) return;

  if (contd) {
  	UIHandler_msg (_("Playing halted."));
  	contd_pos = File_tell (f);
  }
  else {
  	UIHandler_msg ("");
  	UIHandler_time (Time_i2clock (0));
  	UIHandler_size (0, size);
  }

  if (!error) DBuf_close (0);
  if (contd) Task_play_timeout();
  SoundHandler_close();
  Format_close();
  File_close (f);
  closed = 1;
}

/* this is called once in a while by the UI to sync the display and
   do error checking */
int Task_play_timeout()
{
  long seconds;
  static long oldseconds;

  if (error) return 0;
  if (DBuf_rerror()) {
  	error = 1;
  	UIHandler_msg (_("File read error."));
  }
  if (DBuf_werror()) {
  	error = 1;
  	UIHandler_msg (_("Sound write error."));
  }
  
  seconds = written / samp_fac;
  if (seconds != oldseconds) {
  	UIHandler_time (Time_i2clock (seconds));
  	UIHandler_size (read*(b/8), size);
  	oldseconds = seconds;
  }
  
  if (timer > 0 && seconds >= timer)
  	return 0;

//g_print("e=%d r=%d w=%d f=%d\n", error, read, written, read_finished);
  return (error || (written >= read && read_finished))? 0:1;
}

/*------------------------------------------------------------------*/

static long filereader (u32* p, long len)
{
  long l;
  l = Format_get_chunk (p, len);
//g_print("r%d ", l);
  read_finished = (l < len);
  if (l <= 0) return -1;
  read += l;
  convert (p, l);
  return l;
}

static long soundwriter (u32* p, long len)
{
  len = SoundHandler_play_chunk (p, len);
//g_print("w%d ", len);
  written += len;
  return 0;
}


